import java.math.BigInteger;
import java.util.Random;
import java.util.concurrent.atomic.AtomicInteger;

import org.svetovid.io.SvetovidReader;
import org.svetovid.io.SvetovidWriter;

public class StabloIORandom implements StabloIO {

    protected long seed;
    protected int length;
    protected int depth;

    public StabloIORandom() {
        this(System.currentTimeMillis());
    }

    public StabloIORandom(long seed) {
        this(seed, 5, 10);
    }

    public StabloIORandom(long seed, int length, int depth) {
        this.seed = seed;
        this.length = length;
        this.depth = depth;
    }

    public long getSeed() {
        return seed;
    }

    public void setSeed(long seed) {
        this.seed = seed;
    }

    public int getLength() {
        return length;
    }

    public void setLength(int length) {
        this.length = length;
    }

    public int getDepth() {
        return depth;
    }

    public void setDepth(int depth) {
        this.depth = depth;
    }

    @Override
    public Stablo readStablo(SvetovidReader in) {
        Random random = new Random(seed);
        AtomicInteger sequence = new AtomicInteger();
        Stablo stablo = readStablo(random, sequence, length, depth);
        return stablo;
    }

    private Stablo readStablo(Random random, AtomicInteger sequence, int length, int depth) {
    	if (depth <= 0) {
    		return null;
    	}
        int id = sequence.addAndGet(1 + random.nextInt(9));
        String vrednost = new BigInteger(length, random).toString(36);
        Stablo desni = readStablo(random, sequence, length + random.nextInt(5), depth - random.nextInt(5));
        Stablo levi = readStablo(random, sequence, length + random.nextInt(5), depth - random.nextInt(5));
        return new Stablo(id, vrednost, levi, desni);
    }

    @Override
    public void printStablo(SvetovidWriter out, Stablo stablo) {
        throw new UnsupportedOperationException();
    }
}
