import java.util.ArrayList;
import java.util.List;

class Stablo {

    public final int id;
    public String vrednost;
    public Stablo levi;
    public Stablo desni;

    public Stablo(int id) {
        this(id, null);
    }

    public Stablo(int id, String vrednost) {
        this(id, vrednost, null, null);
    }

    public Stablo(int id, String vrednost, Stablo levi, Stablo desni) {
        this.id = id;
        this.vrednost = vrednost;
        this.levi = levi;
        this.desni = desni;
    }

    @Override
    public String toString() {
        return "(" + id + " \"" + vrednost + "\"" + (levi == null ? "" : " " + levi) + (desni == null ? "" : " " + desni) + ")";
    }
}

public class StabloProgram {

    public static void main(String[] args) {

        // Ucitavanje stabla
        StabloIO inIO = new StabloIORandom(12345);
        Stablo stablo = inIO.readStablo(null);

        // Ispisivanje stabla na ekran
        StabloIO outIO = new StabloIOPretty();
        outIO.printStablo(Svetovid.out, stablo);
        Svetovid.out.println();

        // Broj nivoa
        int brojNivoa = brojNivoa(stablo);
        Svetovid.out.println("Broj nivoa:", brojNivoa);
        Svetovid.out.println();

        // Broj nivoa
        int brojElementata = brojElementata(stablo);
        Svetovid.out.println("Broj elementata:", brojElementata);
        Svetovid.out.println();

        // Broj nivoa
        String najduzaVrednost = najduzaVrednost(stablo);
        Svetovid.out.println("Najduza vrednost:", najduzaVrednost);
        Svetovid.out.println();

        // Svi Putevi od korena do listova
        sviPutevi(stablo, new ArrayList<>(brojNivoa));
        Svetovid.out.println();

        // Obrnuto stablo
        Stablo obrnuto = obrni(stablo);
        outIO.printStablo(Svetovid.out, obrnuto);
        Svetovid.out.println();

    }

    private static int brojElementata(Stablo stablo) {
        if (stablo == null) {
            return 0;
        }
        return 1 + brojElementata(stablo.levi) + brojElementata(stablo.desni);
    }

    private static int brojNivoa(Stablo stablo) {
        if (stablo == null) {
            return 0;
        }
        return 1 + Math.max(brojNivoa(stablo.levi), brojNivoa(stablo.desni));
    }

    private static String najduzaVrednost(Stablo stablo) {
        if (stablo == null) {
            return null;
        }
        String rezultat = stablo.vrednost;
        String levaVrednost = najduzaVrednost(stablo.levi);
        String desnaVrednost = najduzaVrednost(stablo.desni);
        if (rezultat == null || (levaVrednost != null && rezultat.length() < levaVrednost.length())) {
            rezultat = levaVrednost;
        }
        if (rezultat == null || (desnaVrednost != null && rezultat.length() < desnaVrednost.length())) {
            rezultat = desnaVrednost;
        }
        return rezultat;
    }

    private static void sviPutevi(Stablo stablo, List<String> put) {
        if (stablo == null) {
            return;
        }
        put.add(stablo.vrednost);
        if ((stablo.levi == null) && (stablo.desni == null)) {
            Svetovid.out.println("Put: " + put);
        }
        sviPutevi(stablo.levi, put);
        sviPutevi(stablo.desni, put);
        put.remove(put.size() - 1);
    }

    private static Stablo obrni(Stablo stablo) {
        if (stablo == null) {
            return null;
        }
        Stablo levi = obrni(stablo.levi);
        Stablo desni = obrni(stablo.desni);
        Stablo obrnuto = new Stablo(stablo.id, stablo.vrednost, desni, levi);
        return obrnuto;
    }
}
